/**********************************************************************/
/* Project Name:  flash_S12Z.c                                        */
/* Last modified: 07/30/2015                                          */
/* By:            b34874                                              */
/**********************************************************************/
/* Description: Flash routines for microcontroller S12Z               */
/* This software is classified as Engineering Sample Software.        */
/**********************************************************************/
#include <hidef.h>           /* common defines and macros */
#include "derivative.h"      /* derivative-specific definitions */
#include "flash_S12Z.h"

//==============================================================================
//PFLASH Send_Command
//==============================================================================
// This function is stored in RAM memory
// in C language:
//  {
//    DisableInterrupts;
//    FSTAT_CCIF = 1;           //launch command
//    while(FSTAT_CCIF == 0){}; //wait for done
//    EnableInterrupts;
//  }
// We cannot simultaneously execute code from the same flash block which we program.
// Therefore, in case when interrupt routines are stored in flash, we disable interrupts, set I bit in CCR
// execute flash commands and then restore the CCR register back again.

// We can execute just simply code in array:
static unsigned char Send_Command[]=
{
  0x04, 0x10, 0xDE, 0x10, 0xED, 0xF0, 0x03, 0x86, 0x02, 0xF0, 0x03, 0x86, 0x00, 0x04, 0x90, 0x05
};

//or we can use linker pragmas for placing this function into RAM:
#pragma CODE_SEG DEFAULT_RAM
void PFLASH_Send_Command(void)
{
   asm PSH CCL;	// 0x10 command
   DisableInterrupts;
   
   FSTAT_CCIF = 1;           //launch command
   while(FSTAT_CCIF == 0){}; //wait for done
  
   asm PUL CCL; // 0x90 command
}
#pragma CODE_SEG DEFAULT

//==============================================================================
//PFLASH_Erase_Verify_Section
//==============================================================================
unsigned char PFLASH_Erase_Verify_Section(unsigned long int address, unsigned int number_of_phrases)
{
  //check if address is aligned (global address [2:0] = 000)
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
  
  while(FSTAT_CCIF == 0){};   //wait if command in progress
  FSTAT = 0x30;               //clear ACCERR and PVIOL
  
  FCCOBIX = 0x02;             //we will write command up to FCCOB2
  FCCOB0HI = 0x03;
  FCCOB0LO = ((address & 0x00FF0000)>>16);
  
  FCCOB1 = (address & 0x0000FFFF);
  
  FCCOB2 = number_of_phrases;
  
  //select one of options for execution critical section
  PFLASH_Send_Command();
  //asm JSR Send_Command;
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  
  //check if phrases are erased and return result
  if(FSTAT_MGSTAT != 0)
    return NON_ERASED;
  else
    return ERASED;
}


//==============================================================================
//PFLASH_Program
//==============================================================================
unsigned char PFLASH_Program(unsigned long int address, unsigned int *ptr)
{
  unsigned char i;
  
  //check if address is aligned (global address [2:0] != 000)
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
  
  //check if the phrase is erased    
//  if((PFLASH_Erase_Verify_Section(address, 1)) == NON_ERASED)
//    return NON_ERASED;  
    
  while(FSTAT_CCIF == 0){};   //wait if command in progress
  FSTAT = 0x30;               //clear ACCERR and PVIOL    
  
  FCCOBIX = 0x05;             //we will write command up to FCCOB5
  FCCOB0HI = 0x06;
  FCCOB0LO = ((address & 0x00FF0000)>>16);
  
  FCCOB1 = (address & 0x0000FFFF);
  
  FCCOB2 = ptr[0];
  FCCOB3 = ptr[1];
  FCCOB4 = ptr[2];
  FCCOB5 = ptr[3];
      
  //select one of options for execution critical section  
  PFLASH_Send_Command();
  //asm JSR Send_Command;
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  else    
    return OK;
}


//==============================================================================
//PFLASH_Erase_Sector
//==============================================================================
unsigned char PFLASH_Erase_Sector(unsigned long int address)
{
  //size of sector is 512B
  
  //check if address is aligned (global address [2:0] != 000)  
  if((address & 0x00000007) != 0)
    return MISALIGNED_ADDRESS; 
    
  while(FSTAT_CCIF == 0){};   //wait if command in progress
  FSTAT = 0x30;               //clear ACCERR and PVIOL
  
  FCCOBIX = 0x01;             //we will write command up to FCCOB1
  FCCOB0HI = 0x0A;
  FCCOB0LO = ((address & 0x00FF0000)>>16);
   
  FCCOB1 = (address & 0x0000FFFF);
  
  //select one of options for execution critical section  
  PFLASH_Send_Command();
  //asm JSR Send_Command;
  
  if((FSTAT & (FSTAT_ACCERR_MASK | FSTAT_FPVIOL_MASK)) != 0)
    return ACCESS_ERROR;
  else    
    return OK;
}

//==============================================================================
//PFLASH_Read_Byte
//==============================================================================
unsigned char PFLASH_Read_Byte(unsigned long int address)
{
  unsigned char data8;
  data8 = *(unsigned char *)address;
  return data8;
}

//==============================================================================
//PFLASH_Read_Word
//==============================================================================
unsigned int PFLASH_Read_Word(unsigned long int address)
{
  unsigned int data16;
  data16 = *(unsigned int *)address;
  return data16;
}

//==============================================================================
//PFLASH_Init
//==============================================================================
void PFLASH_Init(unsigned char fdiv)
{
  while(FSTAT_CCIF == 0){};   //wait if command in progress  
  FCLKDIV = fdiv;             //osc = ? MHz    
}
