/*******************************************************************************
* NXP Semiconductors
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by NXP in this matter are performed AS IS and without 
any warranty. CUSTOMER retains the final decision relative to the total design 
and functionality of the end product. NXP neither guarantees nor will be 
held liable by CUSTOMER for the success of this project.
NXP DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING, 
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR 
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE OR ADVISE SUPPLIED TO THE PROJECT
BY NXP, AND OR NAY PRODUCT RESULTING FROM NXP SERVICES. IN NO EVENT
SHALL NXP BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF
THIS AGREEMENT.

CUSTOMER agrees to hold NXP harmless against any and all claims demands or
actions by anyone on account of any damage, or injury, whether commercial, 
contractual, or tortuous, rising directly or indirectly as a result of an advise
or assistance supplied CUSTOMER in connection with product, services or goods 
supplied under this Agreement.
********************************************************************************
* File      main.c
* Owner     NXA19261
* Version   1.0
* Date      May-11-2017
* Classification   General Business Information
* Brief     Simulating ECC errors at Flash by cumulative write 
********************************************************************************
* Detailed Description:
*
* This example simulates ECC issue by cumulative write into the same Flash area without an erasing.
* 
* The Flash erase operation set all bits into log1. The Flash program operation may keep bit cells in log1 state
* or change it into log0 state, but not in opposite way. 
* 
* The S12Z MCU Flash is protected by 39-Bit ECC Scheme. Every aligned 32bits is protected by additional 7 bits with ECC checksum. 
* The ECC values are not accessible for users. 
* Every Flash reading triggers also ECC check by internal logic. The single bit error in user data may be corrected by ECC checksum.
* The double bit error cannot be corrected.
* 
* The ECC protection is implemented also at flash controller commands and results are signaling by MGSTAT bits. 
* 
* 
* The first case simulates Single-bit ECC error during reading. The MGSTAT bits after the second write are 0b10 
* due to fact that just 1 bit is different during write verification (correctable error)
*
* The second case simulates Single-bit ECC error during reading. The MGSTAT bits after the second write are 0b11 
* due to fact that more than 1 bits are different during write verification (non-correctable error)
* 
* The third case simulates Double-bit ECC error during reading. The MGSTAT bits after the second write are 0b11 
* due to fact that more than 1 bits are different during write verification (non-correctable error)
* 
*
* The flash patterns are selected for highlighted described behavior and they don't have any real meaning.
*
* The cumulative write is not allowed for normal operation!!!
* The code from this example should be used only for design testing - not in production!!! 
* 
* Please, see the prm file. Address range 0xFF0000-0xFF01FF is excluded from default ROM and is used as user flash memory
* The size of sectors is 512 bytes = 64 phrases.
*
* Note: The PFLASH_Program() function was updated - erase verification is skipped 
* 
********************************************************************************
Revision History:
Version  Date         Author    Description of Changes
1.0      May-11-2017 NXA19261  Initial version
*******************************************************************************/
#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "flash_S12Z.h"

//==============================================================================
// Definitions
//============================================================================== 
#define TEST_ADDRESS 0xFF0000

//==============================================================================
// Global variables
//==============================================================================
volatile unsigned int result1, result2, result3;
unsigned int buffer[4];   
volatile unsigned char err;  

//==============================================================================
// Function Prototypes
//============================================================================== 
interrupt VectorNumber_Vflasherr void Flash_ISR(void);
interrupt VectorNumber_Vme void Machine_Exception_ISR(void);

//==============================================================================
// Functions
//============================================================================== 
/*******************************************************************************
 Function Name : Flash_ISR
 Engineer      : NXA19261
 Date          : May-11-2017
 Parameters    : NONE
 Modifies      : NONE
 Returns       : NONE
 Notes         : Flash error interrupt routine.
 Issues        : NONE
 *******************************************************************************/ 
interrupt VectorNumber_Vflasherr void Flash_ISR(void) 
{
  result1 = FSTAT;
  result2 = FERSTAT;
  FERSTAT = FERSTAT; //clear flag
}

/*******************************************************************************
 Function Name : Machine_Exception_ISR
 Engineer      : NXA19261
 Date          : May-11-2017
 Parameters    : NONE
 Modifies      : NONE
 Returns       : NONE
 Notes         : Machine Exception interrupt routine.
 Issues        : The Machine exception do not store registers and return address on stack. 
                 So, we should leave it by MCU reset. See CPU S12Z Reference Manual for more details:
                 http://www.nxp.com/assets/documents/data/en/reference-manuals/S12ZCPU_RM_V1.pdf
                 The code for MCU reset by COP watchdog is commented out in this example code. 
 *******************************************************************************/ 
interrupt VectorNumber_Vme void Machine_Exception_ISR(void)
{  
  result1 = FSTAT;
  result2 = FERSTAT;
  asm NOP;            //Set breakpoint here. 
                      //The Machine exception do not store registers and return address on stack. 
                      //So, we should leave it by MCU reset:
//  CPMUCOP = 0x01;         //Initialize COP watchdog
//  CPMUARMCOP = 0x00;      //write any value except 0x55 or 0xAA cause MCU reset
}


//==============================================================================
//main
//==============================================================================
void main(void) {       
      
  //initial fill data buffer
  buffer[0] = 0xFFFF;
  buffer[1] = 0xFFFF;
  buffer[2] = 0x0000;
  buffer[3] = 0x0000;

                                                         
  PFLASH_Init(0x06);  //default settings:
                      //internal oscillator 1MHz -> bus clk 6.25MHz -> divide by 0x06 
                      //to achieve FCLK approximately 1MHz
 
  FERCNFG = 0x01;     //enable ECC error interrupt
  
  EnableInterrupts;

    
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //Single bit ECC error for read, single bit ECC error for verify
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                  Word0   Word1   ECC
  //  first write     0000    FF00    76
  //  second write    0000    FF40    30
  //  final state     0000    FF00    30
  //  fixed value     0000    FF40
     
  //erase PFlash sector. 
  err = PFLASH_Erase_Sector(TEST_ADDRESS);  
  
  buffer[0] = 0x0000;
  buffer[1] = 0xFF00;   //ECC checksum for 0x0000 FF00	is 0x76
   
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer);   
  
  asm NOP;
  result1 = FSTAT;
  result2 = FERSTAT;
  asm NOP;
  
  //select pattern which will be used for rewrite P-Flash and generating ECC error.
  buffer[1] = 0xFF40;    //ECC checksum for 0x0000 FF40	is 0x30         
  //after writing, it will generate single bit ECC checksum error. FERSTAT_SFDIF is set
  
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer); 
  
  //the result value in Flash is 0xFF00. Since there is single bit ECC error in ECC checksum can be fixed automatically, 
  //the memory window will display 0xFF40 value instead real value in Flash.
  
  asm NOP;
  //MGSTAT=0b10 => Program command verify detects correctable error (we write 0x0000 FF40(0x30), we read 0x0000 FF00(0x30)) 
  //when the memory window will be at TEST_ADDRESS, debugger will read P-Flash and cause Flash_ISR() execution.
  result3 = PFLASH_Read_Word(TEST_ADDRESS+2);
  asm NOP;
  
  
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //Single bit ECC error for read, double bit ECC error for verify
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
  //                  Word0   Word1   ECC
  //  first write     FFFF    DFF4    20
  //  second write    FFFF    FDFF    29
  //  final state     FFFF    DDF4    20
  //  fixed value     FFFF    DFF4
 
  //erase PFlash sector. 
  err = PFLASH_Erase_Sector(TEST_ADDRESS);  
  
  buffer[0] = 0xFFFF;
  buffer[1] = 0xDFF4;   //ECC checksum for 0xFFFF DFF4	is 0x20
   
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer);   
  
  asm NOP;
  result1 = FSTAT;
  result2 = FERSTAT;
  asm NOP;
  
  //select pattern which will be used for rewrite P-Flash and generating ECC error.
  buffer[1] = 0xFDFF;    //ECC checksum for 0xFFFF FDFF	is 0x29         
  //after writing, it will generate single bit ECC checksum error. FERSTAT_SFDIF is set
  
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer); 
  
  //the result value in Flash is 0xDDF4. Since single bit ECC error can be fixed automatically, the memory window will display 0xDFF4 value instead real value in Flash.
  
  asm NOP;
  //MGSTAT=0b11 => Program command verify detects non-correctable error (we write 0xFFFF FDFF(0x29), we read 0xFFFF DDF4(0x20)) 
  //when the memory window will be at TEST_ADDRESS, debugger will read P-Flash and cause Flash_ISR() execution.
  result3 = PFLASH_Read_Word(TEST_ADDRESS+2);
  asm NOP;
  
  
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //Double bit ECC error for read, double bit ECC error for verify
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                  Word0   Word1   ECC
  //  first write     FFFF    FFF4    44
  //  second write    FFFF    CFF     64
  //  final state     FFFF    FCF4    44

  //erase PFlash sector. 
  err = PFLASH_Erase_Sector(TEST_ADDRESS); 
  
  buffer[0] = 0xFFFF;
  buffer[1] = 0xFFF4;  //ECC checksum for 0xFFFF FFF4	is 0x44

   
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer);   
  
  asm NOP;
  result1 = FSTAT;
  result2 = FERSTAT;
  asm NOP;
  
  //select pattern which will be used for rewrite P-Flash and generating ECC error.
  buffer[1] = 0xFCFF;  //ECC checksum for 0xFFFF FCF4	is 0x64           
  //after writing, it will generate double bit ECC checksum error. FERSTAT_DFDIF is set
  
  //Programm TEST_ADDRESS with pattern which is stored in buffer  
  err = PFLASH_Program(TEST_ADDRESS, buffer);   
  
  //The result value in Flash is 0xFCF4. ECC checksum written in Flash stay 0x44 because we may write just log1->log0 and not in opposite. 
  //Since double bit ECC error cannot be fixed, the memory window will display 0xFCF4 value.
  
  asm NOP;
  //MGSTAT=0b11 => Program command verify detects non-correctable error (we write 0xFFFF FCFF(0x64), we read 0xFFFF FCF4(0x44)) 
  //when the memory window will be at TEST_ADDRESS, debugger will read P-Flash and it cause Machine_Exception_ISR execution.
  result3 = PFLASH_Read_Word(TEST_ADDRESS+2);
  asm NOP;

  

  for(;;) {
    __RESET_WATCHDOG();	/* feeds the dog */
    
  } /* loop forever */
  /* please make sure that you never leave main */

}
